@@ -46,6 +46,7 @@ enum
 	FQR_FETCHING_TUPLES = 1L	/* is fetching tuples from db */
 	,FQR_REACHED_EOF = (1L << 1)	/* reached eof */
 	,FQR_HAS_VALID_BASE = (1L << 2)
+	,FQR_NEEDS_SURVIVAL_CHECK = (1L << 3) /* check if the cursor is open */
 };
 
 struct QResultClass_
@@ -158,7 +159,7 @@ enum {
 #define QR_set_aborted(self, aborted_)		( self->aborted = aborted_)
 #define QR_set_haskeyset(self)		(self->flags |= FQR_HASKEYSET)
 #define QR_set_synchronize_keys(self)	(self->flags |= FQR_SYNCHRONIZEKEYS)
-#define QR_set_no_cursor(self)		(self->flags &= ~(FQR_WITHHOLD | FQR_HOLDPERMANENT))
+#define QR_set_no_cursor(self)		(self->flags &= ~(FQR_WITHHOLD | FQR_HOLDPERMANENT | FQR_NEEDS_SURVIVAL_CHECK))
 #define QR_set_withhold(self)		(self->flags |= FQR_WITHHOLD)
 #define QR_set_permanent(self)		(self->flags |= FQR_HOLDPERMANENT)
 #define	QR_set_reached_eof(self)	(self->pstatus |= FQR_REACHED_EOF)
@@ -166,6 +167,8 @@ enum {
 #define	QR_set_no_fetching_tuples(self)	(self->pstatus &= ~FQR_FETCHING_TUPLES)
 #define QR_set_has_valid_base(self)	(self->pstatus |= FQR_HAS_VALID_BASE)
 #define QR_set_no_valid_base(self)	(self->pstatus &= ~FQR_HAS_VALID_BASE)
+#define QR_set_survival_check(self)	(self->pstatus |= FQR_NEEDS_SURVIVAL_CHECK)
+#define QR_set_no_survival_check(self)	(self->pstatus &= ~FQR_NEEDS_SURVIVAL_CHECK)
 #define	QR_inc_num_cache(self) \
 do { \
 	self->num_cached_rows++; \
@@ -195,6 +198,7 @@ do { \
 #define QR_once_reached_eof(self)	((self->pstatus & FQR_REACHED_EOF) != 0)
 #define QR_is_fetching_tuples(self)	((self->pstatus & FQR_FETCHING_TUPLES) != 0)
 #define	QR_has_valid_base(self)		(0 != (self->pstatus & FQR_HAS_VALID_BASE))
+#define	QR_needs_survival_check(self)		(0 != (self->pstatus & FQR_NEEDS_SURVIVAL_CHECK))
 
 #define QR_aborted(self)		(!self || self->aborted)
 #define QR_get_reqsize(self)		(self->rowset_size_include_ommitted)
