@@ -389,6 +389,7 @@ SC_Constructor(ConnectionClass *conn)
 		SC_set_rowset_start(rv, -1, FALSE);
 		rv->current_col = -1;
 		rv->bind_row = 0;
+		rv->from_pos = rv->where_pos = -1;
 		rv->last_fetch_count = rv->last_fetch_count_include_ommitted = 0;
 		rv->save_rowset_size = -1;
 
@@ -852,7 +853,7 @@ inolog("%s statement=%p ommitted=0\n", func, self);
 }
 
 /*
- *	Scan the query wholly or partially (if specifed next_cmd).
+ *	Scan the query wholly or partially (if the next_cmd param specified).
  *	Also count the number of parameters respectviely.
  */
 void
@@ -867,7 +868,8 @@ SC_scanQueryAndCountParams(const char *query, const ConnectionClass *conn,
 	char	tchar, bchar, escape_in_literal = '\0';
 	char	in_literal = FALSE, in_identifier = FALSE,
 		in_dollar_quote = FALSE, in_escape = FALSE,
-		del_found = FALSE;
+		in_line_comment = FALSE, del_found = FALSE;
+	int	comment_level = 0;
 	po_ind_t multi = FALSE;
 	SQLSMALLINT	num_p;
 	encoded_str	encstr;
@@ -926,6 +928,26 @@ SC_scanQueryAndCountParams(const char *query, const ConnectionClass *conn,
 			if (tchar == identifier_quote)
 				in_identifier = FALSE;
 		}
+		else if (in_line_comment)
+		{
+			if (PG_LINEFEED == tchar)
+				in_line_comment = FALSE;
+		}
+		else if (comment_level > 0)
+		{
+			if ('/' == tchar && '*' == sptr[1])
+			{
+				encoded_nextchar(&encstr);
+				sptr++;
+				comment_level++;
+			}
+			else if ('*' == tchar && '/' == sptr[1])
+			{
+				encoded_nextchar(&encstr);
+				sptr++;
+				comment_level--;
+			}
+		}
 		else
 		{
 			if (tchar == '?')
@@ -968,6 +990,24 @@ SC_scanQueryAndCountParams(const char *query, const ConnectionClass *conn,
 			}
 			else if (tchar == identifier_quote)
 				in_identifier = TRUE;
+			else if ('-' == tchar)
+			{
+				if ('-' == sptr[1])
+				{
+					encoded_nextchar(&encstr);
+					sptr++;
+					in_line_comment = TRUE;
+				}
+			}
+			else if ('/' == tchar)
+			{
+				if ('*' == sptr[1])
+				{
+					encoded_nextchar(&encstr);
+					sptr++;
+					comment_level++;
+				}
+			}
 			if (!isspace(tchar))
 				bchar = tchar;
 		}
@@ -1493,14 +1533,17 @@ SC_fetch(StatementClass *self)
 	Int2		num_cols,
 				lf;
 	OID			type;
+	int		atttypmod;
 	char	   *value;
 	ColumnInfoClass *coli;
 	BindInfoClass	*bookmark;
 
 	/* TupleField *tupleField; */
 
-inolog("%s statement=%p ommitted=0\n", func, self);
+inolog("%s statement=%p res=%x ommitted=0\n", func, self, res);
 	self->last_fetch_count = self->last_fetch_count_include_ommitted = 0;
+	if (!res)
+		return SQL_ERROR;
 	coli = QR_get_fields(res);	/* the column info */
 
 	mylog("fetch_cursor=%d, %p->total_read=%d\n", SC_is_fetchcursor(self), res, res->num_total_read);
@@ -1600,8 +1643,8 @@ inolog("%s: stmt=%p ommitted++\n", func, self);
 
 		sprintf(buf, FORMAT_ULEN, SC_get_bookmark(self));
 		SC_set_current_col(self, -1);
-		result = copy_and_convert_field(self, 0, buf,
-			 SQL_C_ULONG, bookmark->buffer + offset, 0,
+		result = copy_and_convert_field(self, 0, PG_UNSPECIFIED, buf,
+			 SQL_C_ULONG, 0, bookmark->buffer + offset, 0,
 			LENADDR_SHIFT(bookmark->used, offset),
 			LENADDR_SHIFT(bookmark->used, offset));
 	}
@@ -1629,8 +1672,9 @@ inolog("%s: stmt=%p ommitted++\n", func, self);
 
 			/* type = QR_get_field_type(res, lf); */
 			type = CI_get_oid(coli, lf);		/* speed things up */
+			atttypmod = CI_get_atttypmod(coli, lf);	/* speed things up */
 
-			mylog("type = %d\n", type);
+			mylog("type = %d, atttypmod = %d\n", type, atttypmod);
 
 			if (SC_is_fetchcursor(self))
 				value = QR_get_value_backend(res, lf);
@@ -1644,7 +1688,7 @@ inolog("curt=%d\n", curt);
 
 			mylog("value = '%s'\n", (value == NULL) ? "<NULL>" : value);
 
-			retval = copy_and_convert_field_bindinfo(self, type, value, lf);
+			retval = copy_and_convert_field_bindinfo(self, type, atttypmod, value, lf);
 
 			mylog("copy_and_convert: retval = %d\n", retval);
 
@@ -1847,7 +1891,8 @@ inolog("get_Result=%p %p %d\n", res, SC_get_Result(self), self->curr_param_resul
 			goto cleanup;
 		}
 	}
-	else if (self->statement_type == STMT_TYPE_SELECT)
+	else if (self->statement_type == STMT_TYPE_SELECT ||
+		 self->statement_type == STMT_TYPE_PROCCALL)
 	{
 		char		fetch[128];
 		const char *appendq = NULL;
@@ -2057,6 +2102,34 @@ inolog("res->next=%p\n", tres);
 	{
 		Int2	io, out;
 		has_out_para = (CountParameters(self, NULL, &io, &out) > 0);
+/*
+ *	I'm not sure if the following REFCIR_SUPPORT stuff is valuable
+ *	or not.
+ */
+#ifdef	REFCUR_SUPPORT
+
+inolog("!!! numfield=%d field_type=%u\n", QR_NumResultCols(res), QR_get_field_type(res, 0));
+		if (!has_out_para &&
+		    0 < QR_NumResultCols(res) &&
+		    PG_TYPE_REFCURSOR == QR_get_field_type(res, 0))
+		{
+			char	fetch[128];
+			int	stmt_type = self->statement_type;
+
+			STR_TO_NAME(self->cursor_name, QR_get_value_backend_text(res, 0, 0));
+			QR_Destructor(res);
+			SC_init_Result(self);
+			SC_set_fetchcursor(self); 
+			qi.result_in = NULL;
+			qi.cursor = SC_cursor_name(self);
+			qi.row_size = ci->drivers.fetch_max;
+			snprintf(fetch, sizeof(fetch), "%s " FORMAT_LEN " in \"%s\"", fetch_cmd, qi.row_size, SC_cursor_name(self));
+			if (0 != (ci->extra_opts & BIT_IGNORE_ROUND_TRIP_TIME))
+				qflag |= IGNORE_ROUND_TRIP;
+			if (res = CC_send_query_append(conn, fetch, &qi, qflag, SC_get_ancestor(self), NULL), NULL != res)
+				SC_set_Result(self, res);
+		}
+#endif	/* REFCUR_SUPPORT */
 	}
 	if (has_out_para)
 	{	/* get the return value of the procedure call */
@@ -2476,7 +2549,11 @@ inolog("!![%d].PGType %u->%u\n", i, PIC_get_pgtype(ipdopts->parameters[i]), CI_g
 				break;
 			case 'B': /* Binary data */
 			case 'D': /* ASCII data */
-				QR_get_tupledata(res, id == 'B');
+				if (!QR_get_tupledata(res, id == 'B'))
+				{
+					rcvend = TRUE;
+					res = NULL;
+				}
 				break;
 			case 'S': /* parameter status */
 				getParameterValues(conn);
