@@ -105,6 +105,7 @@ enum
 	,STMT_TYPE_INSERT
 	,STMT_TYPE_UPDATE
 	,STMT_TYPE_DELETE
+	,STMT_TYPE_WITH
 	,STMT_TYPE_CREATE
 	,STMT_TYPE_ALTER
 	,STMT_TYPE_DROP
@@ -255,11 +256,11 @@ struct StatementClass_
 	SQLLEN		exec_end_row;
 	SQLLEN		exec_current_row;
 
-	char		pre_executing;	/* This statement is prematurely executing */
-	char		inaccurate_result;	/* Current status is PREMATURE but
+	po_ind_t	pre_executing;	/* This statement is prematurely executing */
+	po_ind_t	inaccurate_result;	/* Current status is PREMATURE but
 						 * result is inaccurate */
 	unsigned char	miscinfo;
-	char		updatable;
+	po_ind_t	updatable;
 	SQLLEN		diag_row_count;
 	char		*load_statement; /* to (re)load updatable individual rows */
 	char		*execute_statement; /* to execute the prepared plans */
@@ -327,6 +328,9 @@ do { \
 #define	SC_checked_hasoids(a)	(0 != (a->parse_status & STMT_PARSED_OIDS))
 #define	SC_set_delegate(p, c) (p->execute_delegate = c, c->execute_parent = p)
 
+#define	SC_is_updatable(s)	(0 < ((s)->updatable))
+#define	SC_reset_updatable(s)	((s)->updatable = -1)
+#define	SC_set_updatable(s, b)	((s)->updatable = (b))
 #define	SC_clear_parse_method(s)	((s)->parse_method = 0)
 #define	SC_is_parse_forced(s)	(0 != ((s)->parse_method & 1L))
 #define	SC_set_parse_forced(s)	((s)->parse_method |= 1L)
@@ -384,6 +388,8 @@ enum {
 enum
 {
 	NOT_YET_PREPARED = 0
+	,PREPARING_PERMANENTLY 
+	,PREPARING_TEMPORARILY
 	,PREPARED_PERMANENTLY 
 	,PREPARED_TEMPORARILY
 	,ONCE_DESCRIBED
@@ -423,6 +429,8 @@ enum
 #define SC_unref_CC_error(a)	((a->ref_CC_error) = FALSE)
 #define SC_ref_CC_error(a)	((a->ref_CC_error) = TRUE)
 #define SC_forget_unnamed(a)	(PREPARED_TEMPORARILY == (a)->prepared ? SC_set_prepared(a, ONCE_DESCRIBED) : (void) 0)
+#define SC_returns_rows(a) (STMT_TYPE_SELECT == (a)->statement_type || STMT_TYPE_WITH == (a)->statement_type)
+#define SC_determine_statement_type(a) (STMT_TYPE_SELECT != (a)->statement_type ? (a)->statement_type : ((a)->statement_type = || statement_type((a)->statement)))
 
 
 /* For Multi-thread */
@@ -504,7 +512,8 @@ RETCODE		DiscardStatementSvp(StatementClass *self, RETCODE, BOOL errorOnly);
 
 BOOL		SendParseRequest(StatementClass *self, const char *name,
 			const char *query, Int4 qlen, Int2 num_params);
-BOOL		SendDescribeRequest(StatementClass *self, const char *name);
+BOOL		SyncParseRequest(ConnectionClass *conn);
+BOOL		SendDescribeRequest(StatementClass *self, const char *name, BOOL paramAlso);
 BOOL		SendBindRequest(StatementClass *self, const char *name);
 BOOL		BuildBindRequest(StatementClass *stmt, const char *name);
 BOOL		SendExecuteRequest(StatementClass *stmt, const char *portal, UInt4 count);
